# Components enabled if supported by target architecture:
%ifarch %ix86 x86_64
  %bcond_without gold
%else
  %bcond_with gold
%endif

# (tv) "fix" underlinking issues:
%define _disable_ld_no_undefined 0

%define rc_ 0

# Beware, API docs are ~1G unzipped (as of 3.2)
%bcond_with doc

%define major 3.9
%define minor 1
%define libname %mklibname %name %major
%define libname_devel %mklibname -d %name

%if %{rc_:0}
%define fversion %major.%minor%rc_
%define version %major.%minor~%rc_
%else
%define fversion %major.%minor
%define version %major.%minor
%endif

Name:		llvm
Version:	%version
Release:	%mkrel 6
Summary:	Low Level Virtual Machine (LLVM)
License:	NCSA
Group:		Development/Other
URL:		http://llvm.org
Source0:	http://releases.llvm.org/%{fversion}/llvm-%{fversion}.src.tar.xz
# (tv) fix build with latest python3-sphinx:
Patch0: fix-build.patch
# Fedora Patches
Patch1:	0001-This-code-block-breaks-the-docs-build-http-lab.llvm..patch
Patch2:	0001-fix-docs-2.patch
Patch3:	0001-fix-docs-3.patch
Patch4:	0001-docs-fix-cmake-code-block-warning.patch
Patch5:	0001-cmake-Install-CheckAtomic.cmake-needed-by-lldb.patch
# Fedora Patches (backports from https://github.com/rust-lang/llvm)
Patch47:	rust-lang-llvm-pr47.patch
Patch53:	rust-lang-llvm-pr53.patch
Patch54:	rust-lang-llvm-pr54.patch
Patch55:	rust-lang-llvm-pr55.patch
Patch57:	rust-lang-llvm-pr57.patch
Patch67:	rust-lang-llvm-pr67.patch
Obsoletes:	llvm-ocaml
Requires:	libstdc++-devel
BuildRequires:	cmake
BuildRequires:	python3-sphinx
BuildRequires:	libstdc++-devel
BuildRequires:	libtool
BuildRequires:	zip
BuildRequires:	libffi-devel >= 3.2
BuildRequires:	ncurses-devel
BuildRequires:	chrpath
# Needed for e.g: http://llvm.org/docs/GoldPlugin.html
BuildRequires:	binutils-devel
BuildRequires:  libstdc++-static-devel
# temp force new gcc
BuildRequires:	gcc >= 5.4.0-2

%description
LLVM is a robust system, particularly well suited for developing new mid-level
language-independent analyses and optimizations of all sorts, including those
that require  extensive interprocedural analysis. LLVM is also a great target
for front-end development for conventional or research programming languages,
including those which require compile-time, link-time, or run-time optimization
for effective implementation, proper tail calls or garbage collection.

%package -n %libname
Summary:	LLVM shared libraries
Group:		System/Libraries
Conflicts:	llvm < 3.0-4
Conflicts:	%{libname_devel} < 3.7.1-2
# (tmb) filter out devel provides/requires that gets added because of this symlink
%global __provides_exclude_from ^%{_libdir}/libLLVM-%{version}\\.so$
%define __requires_exclude_from ^%{_libdir}/libLLVM-%{version}\\.so$

%description -n %libname
Shared libraries for the LLVM compiler infrastructure. This is needed by
programs that are dynamically linked against libLLVM.

%package -n %{libname_devel}
Summary:	Development files for LLVM
Group:		Development/Other
Provides:	llvm-devel = %version-%release
Requires:	%libname = %version-%release
Requires:	%name = %version-%release
Conflicts:	llvm < 3.0-7
Conflicts:	%{libname} < 3.7.1-2
Conflicts:	%{_lib}llvm3.0 < 3.0-9

%description -n %{libname_devel}
This package contains the development files for LLVM;

%package static
Summary:	LLVM static libraries
Group:		Development/Other

%description static
Static libraries for the LLVM compiler infrastructure.

%if %{with doc}
%package doc
Summary:	Documentation for LLVM
Group:		Documentation
Requires:	%{name} = %{version}
BuildArch:	noarch
Obsoletes:	llvm-doc-devel

%description doc
Documentation for the LLVM compiler infrastructure.
%endif

%prep
%setup -qn %{name}-%{fversion}.src
%autopatch -p1

%build
# FC compatibility:
#mkdir -p _build
#cd _build
ln -sf build _build

# force off shared libs as cmake macros turns it on.
%cmake \
	-DBUILD_SHARED_LIBS:BOOL=OFF \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_SHARED_LINKER_FLAGS="-Wl,-Bsymbolic -static-libstdc++" \
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64 \
%else
	-DLLVM_LIBDIR_SUFFIX= \
%endif
	\
	-DLLVM_TARGETS_TO_BUILD="X86;AMDGPU;NVPTX;AArch64;ARM;BPF" \
	-DLLVM_ENABLE_LIBCXX:BOOL=OFF \
	-DLLVM_ENABLE_ZLIB:BOOL=ON \
	-DLLVM_ENABLE_FFI:BOOL=ON \
	-DLLVM_ENABLE_RTTI:BOOL=ON \
%if %{with gold}
	-DLLVM_BINUTILS_INCDIR=%{_includedir} \
%endif
	\
	-DLLVM_BUILD_RUNTIME:BOOL=ON \
	\
	-DLLVM_INCLUDE_TOOLS:BOOL=ON \
	-DLLVM_BUILD_TOOLS:BOOL=ON \
	\
	-DLLVM_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_BUILD_TESTS:BOOL=ON \
	\
	-DLLVM_INCLUDE_EXAMPLES:BOOL=ON \
	-DLLVM_BUILD_EXAMPLES:BOOL=OFF \
	\
	-DLLVM_INCLUDE_UTILS:BOOL=ON \
	-DLLVM_INSTALL_UTILS:BOOL=OFF \
	\
	-DLLVM_INCLUDE_DOCS:BOOL=ON \
	-DLLVM_BUILD_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
	-DLLVM_ENABLE_DOXYGEN:BOOL=OFF \
	\
	-DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
	-DLLVM_DYLIB_EXPORT_ALL:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_BUILD_EXTERNAL_COMPILER_RT:BOOL=ON \
	-DLLVM_INSTALL_TOOLCHAIN_ONLY:BOOL=OFF \
	\
%if %mgaversion < 6
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3.4
%else
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3
%endif

%make_build


%if %{with doc}
# Make docs
pushd docs
popd
%endif

%install
cd _build
%make_install

# Since the static libraries are very huge, strip them of debug symbols as well
# (Anssi 08/2012)
strip --strip-debug %{buildroot}%{_libdir}/*.a

%check
exit 0
cd _build
make check-all || :

%files
%license LICENSE.TXT
%{_bindir}/*
%{_mandir}/man1/*
%exclude %{_bindir}/llvm-config
%exclude %{_mandir}/man1/llvm-config.1.*

%files -n %libname
%{_libdir}/libLLVM-%major.so
%{_libdir}/libLLVM-%{version}.so

%files -n %{libname_devel}
%{_bindir}/%{name}-config
%{_includedir}/%{name}
%{_includedir}/%{name}-c
%{_libdir}/BugpointPasses.so
%if %{with gold}
%{_libdir}/LLVMgold.so
%endif
%{_libdir}/libLLVM.so
%{_libdir}/libLTO.so
%{_libdir}/LLVMHello.so
%{_libdir}/cmake/llvm/*.cmake

%doc CREDITS.TXT README.txt
%license LICENSE.TXT
%if %{with doc}
%doc docs/*.css
%doc docs/*.html
%doc docs/_static
%endif
%doc %{_docdir}/%{name}/html
%doc docs/tutorial
%doc docs/CommandGuide
%doc examples

%files static
%{_libdir}/*.a



%changelog
* Wed May 03 2017 akien <akien> 3.9.1-6.mga6
+ Revision: 1098886
- Add P47 and P67 backported from rust-lang fork

* Mon Mar 27 2017 tmb <tmb> 3.9.1-5.mga6
+ Revision: 1094965
- drop broken armv5tl 'buildfix'

* Mon Mar 27 2017 tmb <tmb> 3.9.1-4.mga6
+ Revision: 1094945
- rebuild with new gcc

* Sun Feb 12 2017 akien <akien> 3.9.1-3.mga6
+ Revision: 1085914
- Revert previous commit, some reverse deps like rust explicitely buildconflict llvm-static

* Fri Feb 10 2017 akien <akien> 3.9.1-2.mga6
+ Revision: 1085526
- o Rationale: https://ml.mageia.org/l/arc/dev/2017-02/msg00112.html

  + tv <tv>
    - renumber patches like in FC

* Sat Jan 07 2017 ghibo <ghibo> 3.9.1-1.mga6
+ Revision: 1080449
- Update to release 3.9.1.
- Merge latest 3.9.0 Fedora patches (as well as latest patches for rust).
- Move .cmake files to %%{_libdir}/cmake/llvm/*.cmake, as upstream.
- Drop CppBackend as upstream dropped it.

* Mon Sep 05 2016 dlucio <dlucio> 3.8.1-2.mga6
+ Revision: 1050068
- update libffi BR version
- added support to allow this package to be backported

* Tue Jul 12 2016 tmb <tmb> 3.8.1-1.mga6
+ Revision: 1041233
- update to 3.8.1 (maintenance release)

  + tv <tv>
    - fix color support detection on terminal

* Fri May 13 2016 tv <tv> 3.8.0-7.mga6
+ Revision: 1014737
- patch 0: fix build with latest python3-sphinx

* Mon Apr 25 2016 ngompa <ngompa> 3.8.0-6.mga6
+ Revision: 1006099
- Remove versioned libraries from -devel subpackage

* Fri Mar 25 2016 pterjan <pterjan> 3.8.0-5.mga6
+ Revision: 995390
- Do not ship ThreadPool on armv5tl, it require std::future

  + tv <tv>
    - use new standard make macros
    - use %%license

* Mon Mar 14 2016 tv <tv> 3.8.0-4.mga6
+ Revision: 990520
- submit to core/release

* Wed Mar 09 2016 tv <tv> 3.8.0-3.mga6
+ Revision: 988410
- 3.8.0 final

* Tue Mar 08 2016 tv <tv> 3.8.0-2.mga6
+ Revision: 987334
- clang needs LLVMHello.so
- drop unused support for builds from git & unused macro
- oops forget to reenable debug after testing local builds

* Tue Mar 08 2016 tv <tv> 3.8.0-1.mga6
+ Revision: 987327
- 3.8 RC3
- switch to cmake (losely based on FC work modulo %%cmake differences)
- clean old junk
- clang and the like are split
- split llvm-static out
- add %%check but do not run it yet

* Fri Feb 05 2016 shlomif <shlomif> 3.7.1-4.mga6
+ Revision: 939058
- Enable building LLVMgold.so for -flto/etc.

* Thu Jan 14 2016 tmb <tmb> 3.7.1-3.mga6
+ Revision: 922961
- filter out devel() provides/requires in llvm lib

* Fri Jan 08 2016 tmb <tmb> 3.7.1-2.mga6
+ Revision: 920649
- move libLLVM-<version>.so symlink from devel to main lib (mga#17415, mga#17465)

* Wed Jan 06 2016 tmb <tmb> 3.7.1-1.mga6
+ Revision: 920017
- update to 3.7.1

* Wed Dec 23 2015 tmb <tmb> 3.7.0-4.mga6
+ Revision: 913718
- rebuild with new gcc

* Sun Dec 06 2015 tmb <tmb> 3.7.0-3.mga6
+ Revision: 908619
- rebuild with gcc 5.3

* Wed Sep 23 2015 tmb <tmb> 3.7.0-2.mga6
+ Revision: 882304
- fix 32bit build

  + tv <tv>
    - adjust file list
    - temporary disable doc
    - temporary disable man pages
    - reenable doc+static_analyzer
    - build fixes from FC
    - disable static analyzer (needs switching to cmake)
    - disable doc (needs switching to cmake)
    - drop patch2 (uneeded but breaks build)
    - final release
    - add support for RCs
    - 3.7.0rc3

* Thu Aug 20 2015 tmb <tmb> 3.6.2-5.mga6
+ Revision: 867036
- rebuild with new gcc

* Thu Aug 20 2015 tmb <tmb> 3.6.2-4.mga6
+ Revision: 866694
- rebuild with gcc 5.2.1

* Tue Aug 18 2015 tmb <tmb> 3.6.2-3.mga6
+ Revision: 865426
- rebuild with new glibc

* Sun Aug 02 2015 shlomif <shlomif> 3.6.2-2.mga6
+ Revision: 860846
- Enable RTTI.
- This is per the recommendation here: http://llvm.org/docs/Packaging.html and
  is needed by ponyc, the ponylang compiler.

* Thu Jul 30 2015 shlomif <shlomif> 3.6.2-1.mga6
+ Revision: 859054
- New version 3.6.2
- New version 3.6.1

* Fri Jun 12 2015 tmb <tmb> 3.5.2-1.mga5
+ Revision: 823009
- 3.5.2

* Wed Oct 15 2014 umeabot <umeabot> 3.5.0-3.mga5
+ Revision: 740652
- Second Mageia 5 Mass Rebuild

* Tue Sep 16 2014 umeabot <umeabot> 3.5.0-2.mga5
+ Revision: 681969
- Mageia 5 Mass Rebuild

* Fri Sep 05 2014 tmb <tmb> 3.5.0-1.mga5
+ Revision: 672222
- update filelists
- fix build with gcc-4.9
- update to 3.5.0

* Thu Jul 31 2014 tmb <tmb> 3.4.2-2.mga5
+ Revision: 658764
- rebuild for new gcc

* Mon Jun 16 2014 tmb <tmb> 3.4.2-1.mga5
+ Revision: 637632
- add compiler-rt (mga#13141)
- llvm major is 3.4, not full version
- drop merged patches
- update to 3.4.2

* Thu May 22 2014 tmb <tmb> 3.4.1-5.mga5
+ Revision: 624841
- disable P102 and P104 tu unbreak BS, will be fixed properly for 3.4.2
- fix libclang versioning
- adjust to upstream soname breakage

* Wed May 21 2014 tmb <tmb> 3.4.1-4.mga5
+ Revision: 624783
- Teach Clang to provide ::max_align_t in C11 and C++11 modes
- Headers: Provide an ABI compatible max_align_t when _MSC_VER is defined
- Add front-end infrastructure now address space casts are in LLVM IR
- R600/SI: allow 5 more input SGPRs to a shader
- autoconf: Fix soname for libLLVM-Major.Minor.so

* Tue May 13 2014 tmb <tmb> 3.4.1-3.mga5
+ Revision: 622539
- push to release

* Sun May 11 2014 tmb <tmb> 3.4.1-2.mga5
+ Revision: 621933
- properly detect Mageia (mga#11805)

* Thu May 08 2014 tmb <tmb> 3.4.1-1.mga5
+ Revision: 621249
- update to 3.4.1

* Sun Feb 09 2014 tv <tv> 3.4-1.mga5
+ Revision: 586922
- adjust file list
- adjust file list
- rediff patch
- new release

* Sat Oct 19 2013 umeabot <umeabot> 3.3-2.mga4
+ Revision: 532640
- Mageia 4 Mass Rebuild

* Wed Aug 21 2013 tv <tv> 3.3-1.mga4
+ Revision: 469106
- adjust file list
- new release

* Sun Aug 11 2013 tmb <tmb> 3.2-8.mga4
+ Revision: 465398
- rebuild with new gcc

* Sat Aug 03 2013 tmb <tmb> 3.2-7.mga4
+ Revision: 463049
- test gcc-4.8.2 build

* Fri Aug 02 2013 shlomif <shlomif> 3.2-6.mga4
+ Revision: 462758
- Rebuild for new gcc and libstdc++.

* Thu Feb 28 2013 blino <blino> 3.2-5.mga3
+ Revision: 400669
- add a clang symlink in scan-build directory since scan-build tries to find clang relatively to its real path

* Wed Jan 16 2013 cjw <cjw> 3.2-4.mga3
+ Revision: 388816
- new snapshot from r600 branch (20130109)
  o llvm 3.2 final
  o AMDGPU target renamed to R600

* Sat Jan 12 2013 umeabot <umeabot> 3.2-3.mga3
+ Revision: 358912
- Mass Rebuild - https://wiki.mageia.org/en/Feature:Mageia3MassRebuild

* Sat Dec 29 2012 cjw <cjw> 3.2-2.mga3
+ Revision: 336162
- 3.2 r600 branch (pre-3.2 release) for mesa 9.1pre
- patch1: from 3.2 release svn: fix build with clang 3.2
- enable both x86 and x86-64 targets on x86 and x86-64 hosts

* Mon Dec 24 2012 mitya <mitya> 3.2-1.mga3
+ Revision: 334668
- Fix build
- Cleanup, split apidocs to separate package

  + kamil <kamil>
    - get rid of %%clean
    - remove P1, merged upstream
    - update Version and Release tago to 3.2 and 1
    - new version 3.2

* Sun Oct 14 2012 ennael <ennael> 3.1-8.mga3
+ Revision: 305457
- Documentation group

* Mon Sep 24 2012 mitya <mitya> 3.1-7.mga3
+ Revision: 296968
- Rebuild for GCC 4.7.2

* Thu Sep 06 2012 anssi <anssi> 3.1-6.mga3
+ Revision: 288912
- do not create shared library development symlinks for libclang*.a at all
  since libclang.so only exports a limited set of symbols

* Thu Sep 06 2012 anssi <anssi> 3.1-5.mga3
+ Revision: 288885
- clang-devel: do not create shared library symlinks for those static
  libraries that are actually not included in libclang.so, to fix undefined
  symbol errors when compiling against those (reported by Joseph Wang)
- include libclangTooling.a in libclang.so (shared-tooling.patch from
  upstream)

* Thu Aug 30 2012 fwang <fwang> 3.1-4.mga3
+ Revision: 285683
- promote correct libdir

* Mon Aug 06 2012 luigiwalser <luigiwalser> 3.1-3.mga3
+ Revision: 278987
- rebuild for libffi

* Fri Aug 03 2012 blino <blino> 3.1-2.mga3
+ Revision: 278253
- require llvm in devel package (for tools like llvm-tblgen)

* Thu Aug 02 2012 anssi <anssi> 3.1-1.mga3
+ Revision: 277999
- new version 3.1
- drop llvm-soname.patch, fixed upstream
- re-enable clang
- replace the use of %%exclude with explicit file lists

* Thu Aug 02 2012 anssi <anssi> 3.0-9.mga3
+ Revision: 277981
- fix libification of libllvm3.0 by moving unversioned libraries to the
  devel package
- libify libclang and versionize the shared library
- drop llvm-config patch for dynamic linking, and instead provide
  .so symlinks that match the .a names so that shared linking will
  be enabled with the regular "llvm-config --libs" output
- move clang development libraries to libified clang-devel package
- force build with g++ and gcc instead of buildconflicts
- strip debug symbols from static libraries to make the llvm-devel
  package approx. 30 times smaller
- in preparation of 3.1 release which will be immediately released
  after this package release (the main purpose of this intermediate
  release is to provide a properly libified libllvm3.0):
  o drop gold LTO link patch, fixed upstream in 3.1
  o temporarily disable clang

  + blino <blino>
    - buildconflict with clang < 3.1 since it would fail to build with gcc 4.7 headers

* Wed Jun 27 2012 fwang <fwang> 3.0-8.mga3
+ Revision: 264273
- rebuild for new gcc

* Thu Apr 12 2012 blino <blino> 3.0-7.mga2
+ Revision: 230547
- move libLLVM.so in devel package

* Thu Apr 12 2012 blino <blino> 3.0-6.mga2
+ Revision: 230524
- llvm-devel should require the library package

* Thu Apr 12 2012 fwang <fwang> 3.0-5.mga2
+ Revision: 230344
- add conflicts so that it upgrades correctly

* Thu Apr 12 2012 blino <blino> 3.0-4.mga2
+ Revision: 230340
- split out static libs, headers and config binary in a devel package
- do not package LLVMHello.so (taken from Fedora)
- move sub-libs in library package (but exclude big clang)

* Wed Apr 11 2012 blino <blino> 3.0-3.mga2
+ Revision: 230271
- add clang and enable it by default (most spec bits are copied from Fedora)

  + fwang <fwang>
    - add upstream patch to fix gold link

* Mon Dec 05 2011 fwang <fwang> 3.0-1.mga2
+ Revision: 176830
- update file list
- update file list
- Copy missing file into the expected location
- new version 3.0

* Wed Nov 09 2011 anssi <anssi> 2.9-2.mga2
+ Revision: 165987
- split libLLVM shared library to a library package, allowing
  e.g. DRI drivers to link to it instead of using static linking
- make "llvm-config --libs" print the shared library instead of
  static libraries (config-shared.patch)
- add soname to libLLVM (soname.patch)
- versionize obsoletes
- move the libLLVM shared library to a standard path
- build with libffi support
- build with PIC enabled on all platforms
- drop wrong rpaths from binaries
- add development symlink for the libLLVM shared library
- fix typo in obsoletes

* Wed Jun 22 2011 fwang <fwang> 2.9-1.mga2
+ Revision: 111941
- new version 2.9

* Fri Feb 11 2011 ahmad <ahmad> 2.8-4.mga1
+ Revision: 50381
- imported package llvm

